template <class T>
class GeometricField
:
    public List<T>
{
private:
    fvMesh mesh_;
    IOobject obj_;
public:
    GeometricField(const IOobject& obj, fvMesh& mesh)
    :
        List<T>(mesh.nCells()+2),
        mesh_(mesh),
        obj_(obj)
    {
        readField("internalField");
    }
    fvMesh& mesh()
    {
        return mesh_;
    }
    void readField(const string& fieldDictEntry)
    {
        if(obj_.readOption())
        {
            string firstToken;
            string fileName=obj_.pathName()+"/"+obj_.fileName();
            ifstream is(fileName.c_str());
            is >> firstToken;
            if(firstToken==fieldDictEntry)
            {
                is >> firstToken;
                if(firstToken=="uniform") operator=(is);
                else if(firstToken=="nonUniform") operator>>(is);
            }
            is.close();
        }
    }
    void write()
    {
        if(obj_.writeOption())
        {
            string fileName=obj_.pathName()+"/"+obj_.fileName();
            ofstream os(fileName.c_str());
            os << "internalField nonUniform " << endl;
            for(int i=1;i<this->size()-1;i++) os << this->operator[](i) << endl;
            os.close();
        }
    }
    void bcond()
    {
        this->operator[](0)=-100.0;
        //this->operator[](this->size()-1)=200.0;      // for Dirichlet
        this->operator[](this->size()-1)=this->operator[](this->size()-2);
    }
    ifstream& operator=(ifstream& is)
    {
        T v;
        is >> v;
        for(int i=1;i<this->size()-1;i++) this->operator[](i)=v;
        return is;
    }
    ifstream& operator>>(ifstream& is)
    {
        T v;
        for(int i=1;i<this->size()-1;i++) 
        {
            is >> v;
            this->operator[](i)=v;
        }
        return is;
    }
};

