#include "fixedValueFvPatchFieldFor15th.H"
#include "zeroGradientFvPatchFieldFor15th.H"
#include "fvPatchFieldFor15th.H"

template <class T>
class GeometricField
:
    public List<T>
{
public:
    class Boundary
    :
        public PtrList<fvPatchField<T> >
    {
    private:
        List<string> fvPatchType_;
    public:
        Boundary()
        {
            this->setSize(2);
            fvPatchType_.setSize(2);
        }
        void readField(GeometricField& vf, const string& fieldDictEntry)
        {
            IOobject& obj = vf.obj();
            if(obj.readOption())
            {
                string firstToken;
                string fileName=obj.pathName()+"/"+obj.fileName();
                ifstream is(fileName.c_str());
                while(!is.eof())
                {
                    is >> firstToken;
                    if(firstToken==fieldDictEntry)
                    {
                        for(int iBoundary=0;iBoundary<this->size();iBoundary++)
                        {
                            is >> fvPatchType_[iBoundary];
                            this->set(iBoundary,fvPatchField<T>::New(vf,1,is,fvPatchType_[iBoundary]));
                        }
                    }
                }
                is.close();
            }
        }
        List<string>& fvPatchType()
        {
            return fvPatchType_;
        }
        void evaluate(const double& value)
        {
            for(int i=0;i<this->size();i++) this->operator[](i).evaluate(value);
        }
        ostream& write(ostream& os)
        {
            os << "boundaryField" << endl;
            for(int i=0;i<this->size();i++)
            {
                if(fvPatchType_[i]=="zeroGradient") os << fvPatchType_[i] << endl;
                else if(fvPatchType_[i]=="fixedValue") os << fvPatchType_[i] << " " << this->operator[](i)[0] << endl;
            }
            return os;
        }
        
    };

    GeometricField(const IOobject& obj, fvMesh& mesh)
    :
        List<T>(mesh.nCells()+2),
        mesh_(mesh),
        obj_(obj)
    {
        readField("internalField");
        boundaryField_.readField(*this, "boundaryField");
    }
    fvMesh& mesh()
    {
        return mesh_;
    }
    IOobject& obj()
    {
        return obj_;
    }
    void readField(const string& fieldDictEntry)
    {
        if(obj_.readOption())
        {
            string firstToken;
            string fileName=obj_.pathName()+"/"+obj_.fileName();
            ifstream is(fileName.c_str());
            is >> firstToken;
            if(firstToken==fieldDictEntry)
            {
                is >> firstToken;
                if(firstToken=="uniform") operator=(is);
                else if(firstToken=="nonUniform") operator>>(is);
            }
            is.close();
        }
    }
    void write()
    {
        if(obj_.writeOption())
        {
            string fileName=obj_.pathName()+"/"+obj_.fileName();
            ofstream os(fileName.c_str());
            os << "internalField nonUniform " << endl;
            for(int i=1;i<this->size()-1;i++) os << this->operator[](i) << endl;
            boundaryField_.write(os);
            os.close();
        }
    }
    void correctBoundaryConditions()
    {
        // internalCell has not been defined yet, so this is only valid for zeroGradient outlet
        boundaryField_.evaluate(this->operator[](this->size()-2));
    }
    Boundary& boundaryField()
    {
        return boundaryField_;
    }
    ifstream& operator=(ifstream& is)
    {
        T v;
        is >> v;
        for(int i=1;i<this->size()-1;i++) this->operator[](i)=v;
        return is;
    }
    ifstream& operator>>(ifstream& is)
    {
        T v;
        for(int i=1;i<this->size()-1;i++) 
        {
            is >> v;
            this->operator[](i)=v;
        }
        return is;
    }
private:
    fvMesh mesh_;
    IOobject obj_;
    Boundary boundaryField_;
};

