#include <fstream>
#include <iostream>
#include <cmath>

#define NTST 20
#define DT 0.01
#define DY 1.0
#define DZ 1.0
#define ALPHA 1.0
using namespace std;

#include "List.H"
#include "fvMatrixFor03rd.H"

void init_mesh(List<double>& dx, List<double>& V, List<double>& Af)
{
    for(int i=1;i<dx.size();i++) dx[i]=1.0/double(V.size()-1);
    dx[1]=0.5*dx[1];
    dx[dx.size()-1]=0.5*dx[dx.size()-1];
    for(int i=1;i<V.size();i++)
    {
        Af[i]=DY*DZ;
        V[i]=Af[i]*1.0/double(V.size()-1);
    }
    Af[Af.size()-1]=DY*DZ;
}
void init_phi(List<double>& phi)
{
    for(int i=1;i<phi.size()-1;i++) phi[i]=0.0;
}
void write_phi(List<double> phi)
{
    ofstream os("phi");
    for(int i=1;i<phi.size()-1;i++) os << phi[i] << endl;
    os.close();
}
int main()
{
    int M1=10;
    List<double> phi(M1+2), source(M1+1), dx(M1+2), V(M1+1), Af(M1+2);
    init_mesh(dx,V,Af);
    init_phi(phi);
    for(int nTime=0;nTime<NTST;nTime++)
    {
        fvMatrix phiEqn(phi);
        phiEqn.bcond(phi);
        phiEqn.calc_source(phi,dx,V,Af);
        phiEqn.calc_coef(dx,V,Af,nTime);
        phiEqn.solve(phi,nTime);
    }
    write_phi(phi);
    return 0;
}
