class fvMatrix
{
private:
    List<List<double> > coef_;
    List<double> source_;
public:
    fvMatrix()
    {}
    fvMatrix(List<double> phi)
    {
        coef_.setSize(phi.size()-1);
        for(int i=0;i<coef_.size();i++) coef_[i].setSize(phi.size()-1);
        source_.setSize(phi.size()-1);
        for(int i=1;i<coef_.size();i++) 
        {
            source_[i]=0.0;
            for(int j=1;j<coef_[i].size();j++) coef_[i][j]=0.0;
        }
    }
    void bcond(List<double>& phi)
    {
        phi[0]=-100.0;
        //phi[phi.size()-1]=200.0;	// for Dirichlet
        phi[phi.size()-1]=phi[phi.size()-2];
    }
    void calc_source(List<double>& phi, List<double>& dx, List<double>& V, List<double>& Af)
    {
        for(int i=1;i<source_.size();i++)
        {
            source_[i]=1.0/DT*phi[i]*V[i];
        }
        source_[1]+=ALPHA*Af[1]*phi[0]/dx[1];
    }
    void calc_coef(List<double>& dx, List<double>& V, List<double>& Af, const int& nTime)
    {
        for(int i=1;i<coef_.size();i++)
        {
            coef_[i][i]=1.0*V[i]/DT+ALPHA*(Af[i+1]/dx[i+1]+Af[i]/dx[i]);
            if(i!=1) coef_[i-1][i]=-ALPHA*Af[i]/dx[i];
            if(i!=coef_.size()-1) coef_[i+1][i]=-ALPHA*Af[i+1]/dx[i+1];
        }
        coef_[coef_.size()-1][coef_.size()-1]=1.0*V[coef_.size()-1]/DT+ALPHA*Af[coef_.size()-1]/dx[coef_.size()-1];   // for Neumann at i=M1

        if(nTime==NTST-1)
        {
            ofstream coefFile("MATRIX_COEF.dat");
            for(int j=1;j<coef_.size();j++)
            {
                for(int i=1;i<coef_.size();i++) coefFile << coef_[i][j] << " ";
                coefFile << " | " << source_[j] << endl;
            }
            coefFile.close();
        }
    }
    void solve(List<double>& phi, const int& nTime)
    {
        int nIter=1;
        double residual;
        List<double> oldPhi(phi.size());
        do
        {
            for(int i=1;i<phi.size()-1;i++) oldPhi[i]=phi[i];
            for(int i=1;i<phi.size()-1;i++)
            {
                double sumLHS=0.0;
                for(int iLHS=1;iLHS<oldPhi.size()-1;iLHS++) sumLHS+=coef_[iLHS][i]*oldPhi[iLHS];
                sumLHS-=coef_[i][i]*oldPhi[i];
                phi[i]=(source_[i]-sumLHS)/coef_[i][i];
            }
            residual=0.0;
            for(int i=1;i<phi.size()-1;i++) residual+=fabs(phi[i]-oldPhi[i])/double(phi.size()-2);
            cout << nTime << " : RES = " << residual << ", ITER = " << nIter << endl;
            nIter++;
        } while(residual>0.0001);
    }
};

