class fvMatrix
{
private:
    List<List<double> > coef_;
    List<double> source_;
public:
    fvMatrix()
    {}
    fvMatrix(GeometricField<double> phi)
    {
        coef_.setSize(phi.size()-1);
        for(int i=0;i<coef_.size();i++) coef_[i].setSize(phi.size()-1);
        source_.setSize(phi.size()-1);
        for(int i=1;i<coef_.size();i++) 
        {
            source_[i]=0.0;
            for(int j=1;j<coef_[i].size();j++) coef_[i][j]=0.0;
        }
    }
    fvMatrix(fvMatrix& rfvm)
    {
        coef_.setSize(rfvm.coef().size());
        for(int i=0;i<coef_.size();i++) coef_[i].setSize(rfvm.coef()[i].size());
        source_.setSize(rfvm.source().size());
        for(int i=1;i<coef_.size();i++) 
        {
            source_[i]=rfvm.source()[i];
            for(int j=1;j<coef_[i].size();j++) coef_[i][j]=rfvm.coef()[i][j];
        }
    }
    void solve(GeometricField<double>& phi, const int& nTime)
    {
        int nIter=1;
        double residual;
        List<double> oldPhi(phi.size());
        do
        {
            for(int i=1;i<phi.size()-1;i++) oldPhi[i]=phi[i];
            for(int i=1;i<phi.size()-1;i++)
            {
                double sumLHS=0.0;
                for(int iLHS=1;iLHS<oldPhi.size()-1;iLHS++) sumLHS+=coef_[iLHS][i]*oldPhi[iLHS];
                sumLHS-=coef_[i][i]*oldPhi[i];
                phi[i]=(source_[i]-sumLHS)/coef_[i][i];
            }
            residual=0.0;
            for(int i=1;i<phi.size()-1;i++) residual+=fabs(phi[i]-oldPhi[i])/double(phi.size()-2);
            cout << nTime << " : RES = " << residual << ", ITER = " << nIter << endl;
            nIter++;
        } while(residual>0.0001);
    }
    List<List<double> >& coef()
    {
        return this->coef_;
    }
    List<double>& source()
    {
        return this->source_;
    }
    friend fvMatrix& operator-(fvMatrix& rfvmA, fvMatrix& rfvmB)
    {
        for(int i=1;i<rfvmA.coef().size();i++) 
        {
            rfvmA.source()[i]-=rfvmB.source()[i];
            for(int j=1;j<rfvmA.coef()[i].size();j++) rfvmA.coef()[i][j]-=rfvmB.coef()[i][j];
        }
        return rfvmA;
    }
};

