class fvMatrix
{
private:
    List<List<double> > coef_;
    List<double> source_;
    List<double> internalCoef_;
    List<double> boundaryCoef_;
public:
    fvMatrix()
    {}
    fvMatrix(GeometricField<double> phi)
    {
        coef_.setSize(phi.size()-1);
        for(int i=0;i<coef_.size();i++) coef_[i].setSize(phi.size()-1);
        source_.setSize(phi.size()-1);
        boundaryCoef_.setSize(phi.size()-1);
        internalCoef_.setSize(phi.size()-1);
        for(int i=1;i<coef_.size();i++) 
        {
            source_[i]=0.0;
            boundaryCoef_[i]=0.0;
            internalCoef_[i]=0.0;
            for(int j=1;j<coef_[i].size();j++) coef_[i][j]=0.0;
        }
    }
    fvMatrix(fvMatrix& rfvm)
    {
        coef_.setSize(rfvm.coef().size());
        for(int i=0;i<coef_.size();i++) coef_[i].setSize(rfvm.coef()[i].size());
        source_.setSize(rfvm.source().size());
        boundaryCoef_.setSize(rfvm.source().size());
        internalCoef_.setSize(rfvm.source().size());
        for(int i=1;i<coef_.size();i++) 
        {
            source_[i]=rfvm.source()[i];
            boundaryCoef_[i]=rfvm.boundaryCoef()[i];
            internalCoef_[i]=rfvm.internalCoef()[i];
            for(int j=1;j<coef_[i].size();j++) coef_[i][j]=rfvm.coef()[i][j];
        }
    }
    void solve(GeometricField<double>& phi, const int& nTime)
    {
        int nIter=1;
        double residual;
        List<double> oldPhi(phi.size());
        do
        {
            for(int i=1;i<phi.size()-1;i++) oldPhi[i]=phi[i];
            for(int i=1;i<phi.size()-1;i++)
            {
                double sumLHS=0.0;
                for(int iLHS=1;iLHS<oldPhi.size()-1;iLHS++) 
                {
                    if(i==iLHS) sumLHS+=(coef_[iLHS][i]+internalCoef_[i])*oldPhi[iLHS];
                    else sumLHS+=coef_[iLHS][i]*oldPhi[iLHS];
                }
                sumLHS-=(coef_[i][i]+internalCoef_[i])*oldPhi[i];
                phi[i]=(boundaryCoef_[i]+source_[i]-sumLHS)/(coef_[i][i]+internalCoef_[i]);
            }
            residual=0.0;
            for(int i=1;i<phi.size()-1;i++) residual+=fabs(phi[i]-oldPhi[i])/double(phi.size()-2);
            cout << nTime << " : RES = " << residual << ", ITER = " << nIter << endl;
            nIter++;
        } while(residual>0.0001);
    }
    List<List<double> >& coef()
    {
        return this->coef_;
    }
    List<double>& internalCoef()
    {
        return this->internalCoef_;
    }
    List<double>& source()
    {
        return this->source_;
    }
    List<double>& boundaryCoef()
    {
        return this->boundaryCoef_;
    }
    friend fvMatrix& operator-(fvMatrix& rfvmA, fvMatrix& rfvmB)
    {
        for(int i=1;i<rfvmA.coef().size();i++) 
        {
            rfvmA.source()[i]-=rfvmB.source()[i];
            rfvmA.boundaryCoef()[i]-=rfvmB.boundaryCoef()[i];
            rfvmA.internalCoef()[i]-=rfvmB.internalCoef()[i];
            for(int j=1;j<rfvmA.coef()[i].size();j++) rfvmA.coef()[i][j]-=rfvmB.coef()[i][j];
        }
        return rfvmA;
    }
};

