namespace fvm
{
    fvMatrix& ddt(GeometricField<double>& phi)
    {
        tmp<fvMatrix> tFvMatrix = new fvMatrix(phi);
        fvMatrix& rfvm = tFvMatrix.ref();
        for(int i=1;i<rfvm.source().size();i++)
        {
            rfvm.source()[i]+=1.0/DT*phi[i]*phi.mesh().V()[i];
            rfvm.coef()[i][i]+=1.0*phi.mesh().V()[i]/DT;
        }
        return rfvm;
    }
    fvMatrix& laplacian(GeometricField<double>& phi)
    {
        tmp<fvMatrix> tFvMatrix = new fvMatrix(phi);
        fvMatrix& rfvm = tFvMatrix.ref();
        const int& mSize=rfvm.source().size();
        for(int i=2;i<mSize-1;i++)
        {
            rfvm.source()[i]=0.0;
            rfvm.coef()[i][i]=-ALPHA*(phi.mesh().Af()[i+1]/phi.mesh().dx()[i+1]+phi.mesh().Af()[i]/phi.mesh().dx()[i]);
            rfvm.coef()[i-1][i]=ALPHA*phi.mesh().Af()[i]/phi.mesh().dx()[i];
            rfvm.coef()[i+1][i]=ALPHA*phi.mesh().Af()[i+1]/phi.mesh().dx()[i+1];
        }
        rfvm.source()[1]=0.0;
        rfvm.coef()[1][1]=-ALPHA*(phi.mesh().Af()[2]/phi.mesh().dx()[2]+phi.mesh().Af()[1]/phi.mesh().dx()[1]);
        rfvm.coef()[2][1]=ALPHA*phi.mesh().Af()[2]/phi.mesh().dx()[2];
        rfvm.boundaryCoef()[1]=-ALPHA*phi.mesh().Af()[1]/phi.mesh().dx()[1]*phi.boundaryField()[0][0];

        rfvm.source()[mSize-1]=0.0;
        rfvm.coef()[mSize-1][mSize-1]=-ALPHA*(phi.mesh().Af()[mSize]/phi.mesh().dx()[mSize]+phi.mesh().Af()[mSize-1]/phi.mesh().dx()[mSize-1]);
        rfvm.coef()[mSize-2][mSize-1]=ALPHA*phi.mesh().Af()[mSize-1]/phi.mesh().dx()[mSize-1];
        rfvm.internalCoef()[rfvm.coef().size()-1]=ALPHA*phi.mesh().Af()[rfvm.coef().size()]/phi.mesh().dx()[rfvm.coef().size()];      // for Neumann at i=M1
        return rfvm;
    }
};

